/*
** Copyright (c) 1996, 2002
**      Jeff Forys (jeff@forys.cranbury.nj.us).  All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that: (1) Redistributions of
** source code must retain the above copyright notice, this list of
** conditions and the following disclaimer, (2) Redistributions in
** binary form must reproduce the above copyright notice, this list
** of conditions and the following disclaimer in the documentation
** and/or other materials provided with the distribution, (3) All
** advertising materials mentioning features or use of this software
** must display the following acknowledgment: ``This product includes
** software developed by Jeff Forys (jeff@forys.cranbury.nj.us).'', (4)
** The name of the author may not be used to endorse or promote products
** derived from this software without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
** WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
** MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*/

#ifndef lint
static char rcsid[] = "$Id: Netintf.c,v 1.11 2004/01/24 04:36:57 forys Exp $";
#endif

#include <sys/param.h>
#include <sys/types.h>

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <ctype.h>
#include <pwd.h>

#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>
#include <X11/cursorfont.h>
#include <X11/Xos.h>

#include "bitmaps/netdown"
#include "bitmaps/nettran"
#include "bitmaps/netinit"
#include "bitmaps/netrun"
#include "bitmaps/netup"
#include "bitmaps/neterr"

#include <X11/Xaw/XawInit.h>
#include "NetintfP.h"
#include <X11/Xmu/Drawing.h>

extern char *ProgName;

/* prototypes */
static	void	Destroy _P_((Widget));
static	void	Initialize _P_((Widget, Widget, ArgList, Cardinal *));
static	void	ReDisplay _P_((Widget, XEvent *, Region));
static	void	Realize _P_((Widget, XtValueMask *, XSetWindowAttributes *));
static	Boolean	SetValues _P_((Widget, Widget, Widget, ArgList, Cardinal *));
static	void	clktick _P_((XtPointer, XtIntervalId *));
static	GC	getGC _P_((NetintfWidget));
static	Boolean	netintfchk _P_((NetintfWidget));
static	void	netintfdraw _P_((NetintfWidget));
static	void	netintfupdn _P_((Widget, XEvent *, String *, Cardinal *));
static	Pixmap	mkPixmap _P_((Display *, NetintfWidget, Pixmap, int,
			int *, int *));

/* misc initialization stuff */
static	char tmTab[] = "<ButtonPress>:  ifupdown()";
static	XtActionsRec actionsList[] = { { "ifupdown", netintfupdn }, };
static	Dimension defDim = 48;

static XtResource resources[] = {
	{ XtNwidth,      XtCWidth,      XtRDimension,  sizeof(Dimension),
	  XtOffsetOf(WidgetRec,core.width), XtRDimension, (XtPointer)&defDim },
	{ XtNheight,     XtCHeight,     XtRDimension,  sizeof(Dimension),
	  XtOffsetOf(WidgetRec,core.height), XtRDimension, (XtPointer)&defDim },
	{ XtNforeground, XtCForeground, XtRPixel,      sizeof(Pixel),
	  XtOffsetOf(NetintfRec,netintf.fg_pixel), XtRString, XtDefaultForeground },
	{ XtNifname,     XtCIfname,     XtRString,     sizeof(String),
	  XtOffsetOf(NetintfRec,netintf.ifname), XtRString, NULL },
	{ XtNconfdir,    XtCConfdir,    XtRString,     sizeof(String),
	  XtOffsetOf(NetintfRec,netintf.confdir), XtRString, NULL },
	{ XtNmaxival,    XtCMaxival,    XtRInt,        sizeof(int),
	  XtOffsetOf(NetintfRec,netintf.maxival), XtRString, IfChkmaxStr },
	{ XtNvolume,     XtCVolume,     XtRInt,        sizeof(int),
	  XtOffsetOf(NetintfRec,netintf.volume), XtRString, VolumeStr },
	{ XtNnoif,       XtCNoif,       XtRInt,        sizeof(int),
	  XtOffsetOf(NetintfRec,netintf.noif), XtRString, NoifStr },
	{ XtNfont,       XtCFont,       XtRFontStruct, sizeof(XFontStruct *),
	  XtOffsetOf(NetintfRec,netintf.font), XtRString, XtDefaultFont },
	{ XtNdbglvl,     XtCDbglvl,     XtRInt,        sizeof(int),
	  XtOffsetOf(NetintfRec,netintf.dbglvl), XtRString, DbglvlStr },
	{ XtNdownPixmap, XtCPixmap,     XtRBitmap,     sizeof(Pixmap),
	  XtOffsetOf(NetintfRec,netintf.down.bitmap), XtRString, NULL },
	{ XtNtranPixmap, XtCPixmap,     XtRBitmap,     sizeof(Pixmap),
	  XtOffsetOf(NetintfRec,netintf.tran.bitmap), XtRString, NULL },
	{ XtNinitPixmap, XtCPixmap,     XtRBitmap,     sizeof(Pixmap),
	  XtOffsetOf(NetintfRec,netintf.init.bitmap), XtRString, NULL },
	{ XtNrunPixmap,  XtCPixmap,     XtRBitmap,     sizeof(Pixmap),
	  XtOffsetOf(NetintfRec,netintf.run.bitmap), XtRString, NULL },
	{ XtNupPixmap,   XtCPixmap,     XtRBitmap,     sizeof(Pixmap),
	  XtOffsetOf(NetintfRec,netintf.up.bitmap), XtRString, NULL },
	{ XtNerrPixmap,  XtCPixmap,     XtRBitmap,     sizeof(Pixmap),
	  XtOffsetOf(NetintfRec,netintf.err.bitmap), XtRString, NULL },
};

NetintfClassRec netintfClassRec = {
	{ /* Core Class Structure */
		(WidgetClass)&simpleClassRec,	/* superclass */
		"Netintf",			/* class name */
		sizeof(NetintfRec),		/* widget size */
		XawInitializeWidgetSet,		/* class init routine */
		NULL,				/* dynamic initialization */
		FALSE,				/* class initialized? */
		Initialize,			/* initialize routine */
		NULL,				/* call after init routine */
		Realize,			/* XCreateWindow */
		actionsList,			/* name to procedure map */
		XtNumber(actionsList),		/* number of entries */
		resources,			/* for subclass fields */
		XtNumber(resources),		/* number of entries */
		NULLQUARK,			/* quarks */
		TRUE,				/* compress MotionNotify */
		TRUE,				/* compress Expose */
		TRUE,				/* compress enter/leave */
		FALSE,				/* no VisibilityNotify */
		Destroy,			/* subclass free data routine */
		NULL,				/* resize routine */
		ReDisplay,			/* redisplay routine */
		SetValues,			/* set subclass values */
		NULL,				/* call after SetValues() */
		XtInheritSetValuesAlmost,	/* "almost" geometry reply */
		NULL,				/* call after *Almost() */
		NULL,				/* assign input focus */
		XtVersion,			/* version */
		NULL,				/* no callbacks */
		tmTab,				/* button presses handlers */
		XtInheritQueryGeometry,		/* get preferred geometry */
		XtInheritDisplayAccelerator,	/* display accelerator */
		NULL				/* extension record */
	},
	{ /* simple fields */
		XtInheritChangeSensitive	/* change sensitive */
	},
	{ /* netintf fields */
		0				/* unused */
	}
};
WidgetClass netintfWidgetClass = (WidgetClass)&netintfClassRec;

/*
 * getGC(w)
 *
 * Retrieve various values for a GC.
 */
static GC
getGC(w)
	NetintfWidget w;
{
	XtGCMask vmask;
	XGCValues xgcv;

	vmask = GCForeground | GCBackground | GCFont | GCFunction |
		GCGraphicsExposures;
	xgcv.foreground = w->netintf.fg_pixel;
	xgcv.background = w->core.background_pixel;
	xgcv.font = w->netintf.font->fid;
	xgcv.function = GXcopy;
	xgcv.graphics_exposures = False;
	return XtGetGC((Widget)w, vmask, &xgcv);
}

/*
 * Initialize(req, new, args, nargs)
 *
 * Initialize a new NetintfWidget's "netintf" structure.
 */
static void
Initialize(req, new, args, nargs)
	Widget req;
	Widget new;
	ArgList args;
	Cardinal *nargs;
{
	NetintfWidget w = (NetintfWidget)new;

	if (w->core.width <= 0)
		w->core.width = 1;
	if (w->core.height <= 0)
		w->core.height = 1;

	/*
	 * Initialize default interface name (if not yet specified).
	 * Otherwise, just ensure that interface name has a trailing
	 * unit number (append "0" if the user did not provide one).
	 */
	if (!w->netintf.ifname) {
		w->netintf.ifname = (String)XtMalloc(strlen(IfDefault) + 1);
		strcpy(w->netintf.ifname, IfDefault);
	} else {
		String cp;

		for (cp = w->netintf.ifname; !isdigit(*cp) && *cp != '\0'; cp++)
			;
		if (*cp == '\0') {
			String iftemp;

			iftemp = (String)XtMalloc(strlen(w->netintf.ifname)+2);
			strcat(strcpy(iftemp, w->netintf.ifname), "0");
			XtFree(w->netintf.ifname);
			w->netintf.ifname = iftemp;
		}
	}

	/*
	 * If no configuration directory was specified, use home dir.
	 * If home dir can not be determined, use _PATH_XNETINTF.
	 */
	if (!w->netintf.confdir) {
		struct passwd *pw;

		if ((pw = getpwuid(getuid())) != NULL) {
			w->netintf.confdir = XtMalloc(strlen(pw->pw_dir) + 1);
			strcpy(w->netintf.confdir, pw->pw_dir);
		} else {
			w->netintf.confdir = XtMalloc(strlen(_PATH_XNETINTF)+1);
			strcpy(w->netintf.confdir, _PATH_XNETINTF);
		}
	}

	/*
	 * If no font was specified, use the default.
	 */
	if (w->netintf.font == NULL)
		w->netintf.font = XQueryFont(XtDisplay(w),
			XGContextFromGC(DefaultGCOfScreen(XtScreen(w))));

	/*
	 * Initialize "local state" information.
	 */
	w->netintf.gc = getGC(w);
	SETSTATE(w,IFST_DOWN);
	w->netintf.sockfd = -1;
	w->netintf.wid = XTextWidth(w->netintf.font, w->netintf.ifname,
		strlen(w->netintf.ifname));
	w->netintf.hgt = w->netintf.font->ascent + w->netintf.font->descent;
	w->netintf.ival = 0;
	w->netintf.intid = (XtIntervalId)0;

	w->netintf.down.pixmap = w->netintf.tran.pixmap =
		w->netintf.init.pixmap = w->netintf.run.pixmap =
		w->netintf.up.pixmap = w->netintf.err.pixmap = None;

	if (w->netintf.dbglvl > 0)
		fprintf(stderr, "%s: DEBUG: intf <%s> conf <%s>\n",
		        ProgName, w->netintf.ifname, w->netintf.confdir);
}

/*
 * mkPixmap(dpy, w, bmp, wdep, widp, hgtp)
 *
 * Construct pixmap of depth "wdep" from bitmap "bmp" for display "dpy"
 * and window "w".  Save width and height in "widp" and "hgtp".
 * The new pixmap is returned.
 */
static Pixmap
mkPixmap(dpy, w, bmp, wdep, widp, hgtp)
	Display *dpy;
	NetintfWidget w;
	Pixmap bmp;
	int wdep, *widp, *hgtp;
{
	Window root;
	int x, y;
	unsigned int wid, hgt, bw, dep;
	unsigned long fore, back;

	if (!XGetGeometry(dpy, bmp, &root, &x, &y, &wid, &hgt, &bw, &dep))
		return None;

	*widp = (int)wid;
	*hgtp = (int)hgt;
	fore = w->netintf.fg_pixel;
	back = w->core.background_pixel;
	return XmuCreatePixmapFromBitmap(dpy, w->core.window, bmp,
		wid, hgt, wdep, fore, back);
}

/*
 * Realize(gw, vmask, attr)
 *
 * Prepare to bring our window into existence (allocate bitmaps, etc).
 */
static void
Realize(gw, vmask, attr)
	Widget gw;
	XtValueMask *vmask;
	XSetWindowAttributes *attr;
{
	NetintfWidget w = (NetintfWidget)gw;
	Display *dpy = XtDisplay(w);
	int depth = w->core.depth;

	*vmask |= (CWBitGravity | CWCursor);
	attr->bit_gravity = ForgetGravity;
	attr->cursor = XCreateFontCursor(dpy, XC_top_left_arrow);

	(*netintfWidgetClass->core_class.superclass->core_class.realize)
		(gw, vmask, attr);

	if (w->netintf.down.bitmap == None)
		w->netintf.down.bitmap =
			XCreateBitmapFromData(dpy, w->core.window,
			 (char *)netdown_bits, netdown_width, netdown_height);
	if (w->netintf.tran.bitmap == None)
		w->netintf.tran.bitmap =
			XCreateBitmapFromData(dpy, w->core.window,
			 (char *)nettran_bits, nettran_width, nettran_height);
	if (w->netintf.init.bitmap == None)
		w->netintf.init.bitmap =
			XCreateBitmapFromData(dpy, w->core.window,
			 (char *)netinit_bits, netinit_width, netinit_height);
	if (w->netintf.run.bitmap == None)
		w->netintf.run.bitmap =
			XCreateBitmapFromData(dpy, w->core.window,
			 (char *)netrun_bits, netrun_width, netrun_height);
	if (w->netintf.up.bitmap == None)
		w->netintf.up.bitmap =
			XCreateBitmapFromData(dpy, w->core.window,
			 (char *)netup_bits, netup_width, netup_height);
	if (w->netintf.err.bitmap == None)
		w->netintf.err.bitmap =
			XCreateBitmapFromData(dpy, w->core.window,
			 (char *)neterr_bits, neterr_width, neterr_height);

	w->netintf.down.pixmap = mkPixmap(dpy, w, w->netintf.down.bitmap,
		depth, &w->netintf.down.width, &w->netintf.down.height);
	w->netintf.tran.pixmap = mkPixmap(dpy, w, w->netintf.tran.bitmap,
		depth, &w->netintf.tran.width, &w->netintf.tran.height);
	w->netintf.init.pixmap = mkPixmap(dpy, w, w->netintf.init.bitmap,
		depth, &w->netintf.init.width, &w->netintf.init.height);
	w->netintf.run.pixmap = mkPixmap(dpy, w, w->netintf.run.bitmap,
		depth, &w->netintf.run.width, &w->netintf.run.height);
	w->netintf.up.pixmap = mkPixmap(dpy, w, w->netintf.up.bitmap,
		depth, &w->netintf.up.width, &w->netintf.up.height);
	w->netintf.err.pixmap = mkPixmap(dpy, w, w->netintf.err.bitmap,
		depth, &w->netintf.err.width, &w->netintf.err.height);

	w->netintf.intid =
		XtAppAddTimeOut(XtWidgetToApplicationContext((Widget)w),
		 w->netintf.ival * 1000, clktick, (XtPointer)w);
}

/*
 * Destroy(gw)
 *
 * Free up memory associated with a NetintfWidget's "netintf" structure.
 * Also disarm the Xt interval timer.
 */
static void
Destroy(gw)
	Widget gw;
{
	NetintfWidget w = (NetintfWidget)gw;
	Display *dpy = XtDisplay(gw);

	if (w->netintf.intid)
		XtRemoveTimeOut(w->netintf.intid);
	XtFree(w->netintf.ifname);
	XtFree(w->netintf.confdir);
	XtReleaseGC(gw, w->netintf.gc);
#define	freebpmap(map)							    \
	if (w->netintf.map.bitmap) XFreePixmap(dpy, w->netintf.map.bitmap); \
	if (w->netintf.map.pixmap) XFreePixmap(dpy, w->netintf.map.pixmap)
	freebpmap(down);
	freebpmap(tran);
	freebpmap(init);
	freebpmap(run);
	freebpmap(up);
	freebpmap(err);
#undef	freebpmap
}

/*
 * ReDisplay(gw, event, region)
 *
 * Redraw the NetintfWidget window.
 */
static void
ReDisplay(gw, event, region)
	Widget gw;
	XEvent *event;
	Region region;
{
	netintfdraw((NetintfWidget)gw);
}

/*
 * SetValues(gcur, greq, gnew, args, nargs)
 *
 * If subclass values in "gcur" and "gnew" differ, return TRUE.
 * Otherwise, return FALSE.
 */
static Boolean
SetValues(gcur, greq, gnew, args, nargs)
	Widget gcur;
	Widget greq;
	Widget gnew;
	ArgList args;
	Cardinal *nargs;
{
	NetintfWidget cur = (NetintfWidget)gcur;
	NetintfWidget new = (NetintfWidget)gnew;
	Boolean redisplay = FALSE;

	if (cur->netintf.ival != new->netintf.ival) {
		if (cur->netintf.intid)
			XtRemoveTimeOut(cur->netintf.intid);
		new->netintf.intid =
			XtAppAddTimeOut(XtWidgetToApplicationContext(gnew),
			new->netintf.ival * 1000, clktick, (XtPointer)gnew);
	}

	if (cur->netintf.font != new->netintf.font)
		redisplay = TRUE;

	if (cur->netintf.fg_pixel != new->netintf.fg_pixel ||
	    cur->core.background_pixel != new->core.background_pixel) {
		XtReleaseGC(gcur, cur->netintf.gc);
		new->netintf.gc = getGC(new);
		redisplay = TRUE;
	}

	return(redisplay);
}

/*
 * NetintfExit(gw)
 *
 * Bring down interface associated with NetintfWidget "gw".
 */
void
NetintfExit(gw)
	Widget gw;
{
	NetintfWidget w = (NetintfWidget)gw;

	NetIfDn(w);
}

/*
 * netintfdraw(w)
 *
 * Redraw window.
 */
static void
netintfdraw(w)
	NetintfWidget w;
{
	register Display *dpy = XtDisplay(w);
	register Window win = XtWindow(w);
	GC gc = w->netintf.gc;
	Pixel back = w->core.background_pixel;
	struct _ifimage *im;
	int x, y;

	switch(GETSTATE(w)) {
	    case IFST_DOWN:
		im = &w->netintf.down;
		break;
	    case IFST_INIT:
		im = &w->netintf.init;
		break;
	    case IFST_RUN:
		im = &w->netintf.run;
		break;
	    case IFST_UP:
		im = &w->netintf.up;
		break;
	    case IFST_TOINIT:
	    case IFST_TODOWN:
		im = &w->netintf.tran;
		break;
	    case IFST_ERR:
	    default:
		im = &w->netintf.err;
		break;
	}

	x = (((int)w->core.width) - im->width) / 2;
	y = (((int)w->core.height) - im->height) / 2;
	XSetWindowBackground(dpy, win, back);
	XClearWindow(dpy, win);
	XCopyArea(dpy, im->pixmap, win, gc, 0, 0, im->width, im->height, x, y);

	XDrawImageString(dpy, win, gc,
		x + im->width - w->netintf.wid, y + im->height - 3 /* border */,
		w->netintf.ifname, strlen(w->netintf.ifname));
}

/*
 * netintfdraw(w)
 *
 * Called when a key is clicked in the window.  If interface is up (or
 * coming up), execute command to bring it down.  If interface is down,
 * execute command to bring it up.
 */
static void
netintfupdn(gw, event, param, nparam)
	Widget gw;
	XEvent *event;
	String *param;
	Cardinal *nparam;
{
	NetintfWidget w = (NetintfWidget)gw;

	if (w->netintf.intid)
		XtRemoveTimeOut(w->netintf.intid);

	/*
	 * If in transition to INIT or DOWN state, ignore any state change
	 * requests.  This prevents ugly race conditions when someone pounds
	 * on the icon.
	 */
	if (GETSTATE(w) == IFST_TOINIT || GETSTATE(w) == IFST_TODOWN) {
		XBell(XtDisplay(w), w->netintf.volume);
		goto ignore;
	}

	(void) netintfchk(w);

	/*
	 * Bring the interface up or down, depending on the current state.
	 */
	switch(GETSTATE(w)) {
	    case IFST_DOWN:
		NetIfUp(w);
		if (GETSTATE(w) != IFST_ERR)
			SETSTATE(w,IFST_TOINIT);
		break;
	    case IFST_INIT:
	    case IFST_RUN:
	    case IFST_UP:
		NetIfDn(w);
		if (GETSTATE(w) != IFST_ERR)
			SETSTATE(w,IFST_TODOWN);
		break;
	    default:
		break;
	}

	netintfdraw(w);
ignore:
	w->netintf.ival = IfChkinc;

	w->netintf.intid =
		XtAppAddTimeOut(XtWidgetToApplicationContext((Widget)w),
			w->netintf.ival * 1000, clktick, w);
}

/*
 * clktick(client_data, id)
 *
 * Xt interval timer has gone off.  Check interface for "client_data"
 * and rearm the timer.
 */
static void
clktick(client_data, id)
	XtPointer client_data;
	XtIntervalId *id;
{
	NetintfWidget w = (NetintfWidget)client_data;

	if (netintfchk(w)) {
		w->netintf.ival = IfChkinc;
	} else {
		w->netintf.ival += IfChkinc;
		if (w->netintf.ival > w->netintf.maxival)
			w->netintf.ival = w->netintf.maxival;
	}

	w->netintf.intid =
		XtAppAddTimeOut(XtWidgetToApplicationContext((Widget)w),
		 w->netintf.ival * 1000, clktick, client_data);
}

/*
 * netintfchk(w)
 *
 * Determine current state of interface.  If the state has changed,
 * update the display with the new bitmap.
 * Return TRUE if interface changed state, FALSE otherwise.
 */
static Boolean
netintfchk(w)
	NetintfWidget w;
{
	NetintfState ifstate = NetIfState(w);

	/*
	 * If interface state has changed, update the display.
	 * Further, if the new state is either UP or DOWN, execute
	 * a user-specified command.
	 */
	if (GETSTATE(w) != ifstate) {
		w->netintf.ival = 0;
		SETSTATE(w,ifstate);
		netintfdraw(w);
		switch (ifstate) {
		    case IFST_UP:
			XBell(XtDisplay(w), w->netintf.volume);
			NetIfExecUp(w);
			break;
		    case IFST_DOWN:
			NetIfExecDn(w);
			break;
		    default:
			break;
		}
		return TRUE;
	}

	return FALSE;
}
