.\"
.\" Copyright (c) 1996, 2002
.\"	Jeff Forys (jeff@forys.cranbury.nj.us).  All rights reserved.
.\"
.\" Redistribution and use in source and binary forms, with or without
.\" modification, are permitted provided that: (1) Redistributions of
.\" source code must retain the above copyright notice, this list of
.\" conditions and the following disclaimer, (2) Redistributions in
.\" binary form must reproduce the above copyright notice, this list
.\" of conditions and the following disclaimer in the documentation
.\" and/or other materials provided with the distribution, (3) All
.\" advertising materials mentioning features or use of this software
.\" must display the following acknowledgment: ``This product includes
.\" software developed by Jeff Forys (jeff@forys.cranbury.nj.us).'', (4)
.\" The name of the author may not be used to endorse or promote products
.\" derived from this software without specific prior written permission.
.\"
.\" THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
.\" WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
.\" MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
.\"
.\" $Id: xnetintf.man,v 1.10 2004/01/24 04:36:57 forys Exp $
.\"
.\" xnetintf - control a network interface under X Windows
.\"
.TH XNETINTF 1 "January 23, 2004" "" "Local UNIX Programmer's Manual"
.UC 4
.SH NAME
xnetintf \- control a network interface with X Windows
.SH SYNOPSIS
.B xnetintf
[\-\fItoolkitopts\fP] [\-maxival <secs>] [\-volume <pct>]
.br
         [\-ifname <intf>] [\-confdir <dir>] [\-noif <v>] [\-dbglvl <l>]
.br
.B xnetintf
\-c (up|down|stat)
         [\-ifname <intf>] [\-confdir <dir>] [\-noif <v>] [\-dbglvl <l>]
.br
.SH DESCRIPTION
.I Xnetintf
manages a network interface.  Upon receiving a mouse event,
.I xnetintf
initiates a command to toggle the interface state.
An interface marked \fBdown\fP would begin the transition to
\fBup\fP, while one marked \fBup\fP (or coming up) would being
the transition to \fBdown\fP.
Mouse events are limited to no more than one per second.
.PP
.I Xnetintf
displays the name of the interface in the window, and a series
of bitmaps depicting current interface state.  When
an interface is \fBup\fP, wires connecting a pair of telephone
poles is displayed (the other bitmaps build to this).
Broken telephone poles (and disconnected wires) indicate
a problem with the specified interface.
.PP
It is sometimes useful to be able to run
.I xnetintf
from the command line (i.e. without X Windows).  When the \fI-c\fP
option is present, the next argument specifies the action to perform
on a given interface.  In command line mode,
.I xnetintf
will report status to the terminal until the transition completes
or failure is detected.  Note that, when running in command line mode,
X resources are \fBnot\fP consulted.
.SH OPTIONS
.PP
.I Xnetintf
accepts all standard X Toolkit command line options, as well as
the following:
.IP \-ifname\ <intf>
Name of interface to manage (default is "ppp0").
.IP \-confdir\ <dir>
Location of configuration directory.
.IP \-maxival\ <secs>
Maximum length of time that will pass before rechecking interface
state (default is 30 seconds).  After a state change,
interface checks become more frequent for a period of time.
.IP \-volume\ <pct>
How loud to sound the bell when an interface comes \fBup\fP, or on error.
.IP \-c\ (up|down|stat)
Run in command line mode (see above).
.IP \-noif\ <v>
By default, a non-existent interface is treated as an error.
Specifying a non-zero value for \fIv\fP equates a non-existent
interface to one that is simply down.  This option is necessary
for interfaces that only materialize on demand.
.IP \-dbglvl\ <l>
Set debug level.  Higher integer values for \fIl\fP correspond to more
debug output.
.SH "CONFIGURATION FILE"
Due to the wide variety of methods for changing interface state,
.I xnetintf
employs a per-interface configuration file.
The configuration file consists of 5 commands, one per line:

   1) bring the interface \fBup\fP,
   2) return 0 if interface still being initialized,
   3) execute after the interface transitions to \fBup\fP,
   4) bring the interface \fBdown\fP, and
   5) execute after the interface transitions to \fBdown\fP.
.PP
The configuration file may be no larger than 8192 bytes.  Lines beginning
with a pound sign (``#'') are ignored.  Blank lines are treated as
do-nothing commands.  The configuration file is reread
whenever a command is to be executed.  Finally, to avoid security
risks, commands are executed using
.I execve(2),
therefore you
.B must
specify the full path to the command (or shell) you wish to execute.
Here is a sample configuration file for a PPP interface:

    #
    # 1: Bring interface up.
    /usr/local/lib/xnetintf/ifsuex -u ppp0
    #
    # 2: Check if interface is still trying to come up.
    /usr/local/bin/skill -n -u root pppd
    #
    # 3: Do nothing after interface comes up
    
    #
    # 4: Bring interface down.
    /usr/local/lib/xnetintf/ifsuex -d ppp0
    #
    # 5: Execute this after interface comes down.
    sh /usr/local/lib/xnetintf/.ifpppdn
.PP
If no default configuration file is specified, the user's home directory will
be searched, followed by whatever _PATH_XNETINTF pointed to at compile time.
The name of the per-interface configuration file is of
the form ".if<intf>[<unit>]".  If "<unit>" is not specified, "0" is
assumed (e.g., configuration files named ".ifppp" and ".ifppp0"
will both refer to interface "ppp0").  Additionally, for machines that support
multiple interfaces of the same type, unique links should be created
for common commands (e.g., "pppd" might become "pppd0", "pppd1", etc).
This makes it possible to search for and kill a specific process.
.PP
.SH "EXIT STATUS"
Exit status is normally 0; if the user makes a mistake, 1 is returned.
If a system error occurs, exit status is 2.
.SH "FILES"
/dev/ppp0               - default network interface
.br
/usr/local/lib/xnetintf - default configuration directory
.br
/usr/local/lib/X11/app-defaults/XNetintf - resources
.SH "SEE ALSO"
X(1), ifconfig(1), ifsuex(1), ln(1), skill(1),
.br
execve(2), ioctl(2)
.SH "AUTHOR"
Jeff Forys
.SH "BUGS"
When dealing with IP over phone lines, it's possible that a network
interface is down even though
.I xnetintf
reports that it is up.  This problem can occur when a phone line is
disconnected, where the operating system never learns that the interface
has gone down; the SIOCGIFFLAGS ioctl() continues to report that
the interface is up.  While not a problem with
.I xnetintf
there are two ways to deal with this situation.  If you suspect this has
happened, just click on the bitmap to officially bring the interface down
(i.e. to put it back in a known state).  Alternately, when an interface
comes up, you could initiate a periodic "ping"; if the remote host stops
responding, the "ping" script should use the command-line option to
.I xnetintf
to bring the interface down (`xnetintf -c down ...').
.PP
Currently, nothing prohibits multiple users from using
.I xnetintf
to control the same interface.
